﻿using PInvoke;
using System;
using System.Runtime.InteropServices;

namespace DerpingDrivers.Util
{
    public static class CodeIntegrityHelper
    {
        [UnmanagedFunctionPointer(CallingConvention.StdCall)]
        private delegate Int32 NtQuerySystemInformation(
            UInt32 SystemInformationClass,
            IntPtr SystemInformation,
            UInt32 SystemInformationLength,
            out UInt32 ReturnLength);

        [StructLayout(LayoutKind.Sequential)]
        private struct SYSTEM_CODEINTEGRITY_INFORMATION
        {
            public UInt32 Length;
            public UInt32 CodeIntegrityOptions;
        };

        public static bool IsTestSignEnabled
        {
            get
            {
                var pIntegrity = Marshal.AllocHGlobal(Marshal.SizeOf<SYSTEM_CODEINTEGRITY_INFORMATION>());

                try
                {
                    var fptr = Kernel32.GetProcAddress(Kernel32.GetModuleHandle("ntdll.dll"),
                        "NtQuerySystemInformation");

                    var ntQuerySystemInformation =
                        Marshal.GetDelegateForFunctionPointer<NtQuerySystemInformation>(fptr);

                    SYSTEM_CODEINTEGRITY_INFORMATION integrity;
                    integrity.Length = (uint)Marshal.SizeOf<SYSTEM_CODEINTEGRITY_INFORMATION>();
                    integrity.CodeIntegrityOptions = 0;
                    
                    Marshal.StructureToPtr(integrity, pIntegrity, false);

                    var status = ntQuerySystemInformation(
                        103,
                        pIntegrity,
                        integrity.Length,
                        out _
                    );

                    integrity = Marshal.PtrToStructure<SYSTEM_CODEINTEGRITY_INFORMATION>(pIntegrity);

                    return ((integrity.CodeIntegrityOptions & /* CODEINTEGRITY_OPTION_TESTSIGN */ 0x02) != 0);
                }
                finally
                {
                    Marshal.FreeHGlobal(pIntegrity);
                }
            }
        }
    }
}